@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -19,6 +19,7 @@
 #include <engine/locking.h>
 
 #include <config.h>
+#include <utilities.h>
 
 using namespace BoCA;
 using namespace BoCA::AS;
@@ -63,7 +64,7 @@ using namespace BoCA::AS;
 	}
 #endif
 
-freac::ConvertWorker::ConvertWorker(const BoCA::Config *iConfiguration, Int iConversionID)
+freac::ConvertWorker::ConvertWorker(const BoCA::Config *iConfiguration, Int iConversionID) : workSignal(1)
 {
 	configuration	= iConfiguration;
 
@@ -87,6 +88,8 @@ freac::ConvertWorker::ConvertWorker(const BoCA::Config *iConfiguration, Int iCon
 	cancel		= False;
 	quit		= False;
 
+	workSignal.Wait();
+
 	threadMain.Connect(&ConvertWorker::Perform, this);
 }
 
@@ -99,9 +102,11 @@ Int freac::ConvertWorker::Perform()
 	Bool	 enableParallel	 = configuration->GetIntValue(Config::CategoryResourcesID, Config::ResourcesEnableParallelConversionsID, Config::ResourcesEnableParallelConversionsDefault);
 	Int	 numberOfThreads = configuration->GetIntValue(Config::CategoryResourcesID, Config::ResourcesNumberOfConversionThreadsID, Config::ResourcesNumberOfConversionThreadsDefault);
 
-	while (!quit)
+	while (!Threads::Access::Value(quit))
 	{
-		if (idle) { S::System::System::Sleep(1); continue; }
+		workSignal.Wait();
+
+		if (Threads::Access::Value(quit)) break;
 
 		/* Do not limit parallel CD ripping jobs in automatic mode.
 		 */
@@ -111,15 +116,15 @@ Int freac::ConvertWorker::Perform()
 		 */
 		if (allocateThread) Locking::AllocateThread();
 
-		if (Convert() != Success()) error = True;
+		if (Convert() != Success()) Threads::Access::Set(error, True);
 
 		if (allocateThread) Locking::FreeThread();
 
 		/* Return to waiting state.
 		 */
-		idle	= True;
-		waiting	= True;
-		cancel	= False;
+		Threads::Access::Set(idle, True);
+		Threads::Access::Set(waiting, True);
+		Threads::Access::Set(cancel, False);
 	}
 
 	return Success();
@@ -174,7 +179,7 @@ Int freac::ConvertWorker::Convert()
 	String		 encodeChecksum;
 	String		 verifyChecksum;
 
-	for (Int step = 0; step < conversionSteps && !cancel; step++)
+	for (Int step = 0; step < conversionSteps && !Threads::Access::Value(cancel); step++)
 	{
 		/* Setup file names.
 		 */
@@ -250,8 +255,10 @@ Int freac::ConvertWorker::Convert()
 
 				onReportWarning.Emit(i18n->TranslateString("Skipped verification due to non existing output file: %1", "Messages").Replace("%1", inFile.GetFileName()));
 
+				String	 basePath = Utilities::GetOutputBasePath(configuration, trackToConvert, inFileName);
+
 				log->Lock();
-				log->Write(String("    Skipping verification due to non existing output file: ").Append(inFileName), MessageTypeWarning);
+				log->Write(String("    Skipping verification due to non existing output file: ").Append(Utilities::FormatFileNameForLogging(inFileName, basePath)), MessageTypeWarning);
 				log->Write(NIL);
 				log->Release();
 
@@ -292,8 +299,10 @@ Int freac::ConvertWorker::Convert()
 										  .Append("Output format: %5 Hz, %6 bit, %7 channels"), "Messages").Replace("%1", inFile.GetFileName()).Replace("%2", String::FromInt(format.rate)).Replace("%3", String::FromInt(format.bits)).Replace("%4", String::FromInt(format.channels))
 																						       .Replace("%5", String::FromInt(outFormat.rate)).Replace("%6", String::FromInt(outFormat.bits)).Replace("%7", String::FromInt(outFormat.channels)));
 
+					String	 basePath = Utilities::GetOutputBasePath(configuration, trackToConvert, inFileName);
+
 					log->Lock();
-					log->Write(String("    Skipping verification due to format mismatch: ").Append(inFileName), MessageTypeWarning);
+					log->Write(String("    Skipping verification due to format mismatch: ").Append(Utilities::FormatFileNameForLogging(inFileName, basePath)), MessageTypeWarning);
 					log->Write(NIL);
 					log->Release();
 
@@ -407,7 +416,7 @@ Int freac::ConvertWorker::Convert()
 
 		/* Verify input.
 		 */
-		if (!cancel && verify) VerifyInput(inFileName, verifier);
+		if (!Threads::Access::Value(cancel) && verify) VerifyInput(inFileName, verifier);
 
 		/* Get CRC checksums when ripping.
 		 */
@@ -450,13 +459,13 @@ Int freac::ConvertWorker::Convert()
 
 		/* Delete output file if it doesn't look sane.
 		 */
-		if (outFile.GetFileSize() <= 0 || cancel || error) outFile.Delete();
+		if (outFile.GetFileSize() <= 0 || Threads::Access::Value(cancel) || error) outFile.Delete();
 
 		/* Delete intermediate file in non-on-the-fly mode.
 		 */
 		if (conversionStep == ConversionStepEncode)
 		{
-			if (!keepWaveFiles || cancel || error) inFile.Delete();
+			if (!keepWaveFiles || Threads::Access::Value(cancel) || error) inFile.Delete();
 
 			if (String(inFile).EndsWith(".temp.wav")) inFile = String(inFile).Head(String(inFile).Length() - 9);
 		}
@@ -496,7 +505,7 @@ Int freac::ConvertWorker::Convert()
 
 		/* Revert to waiting state when there are more steps left.
 		 */
-		if (step < conversionSteps - 1) waiting = True;
+		if (step < conversionSteps - 1) Threads::Access::Set(waiting, True);
 
 		/* Report finished conversion.
 		 */
@@ -538,17 +547,17 @@ Int64 freac::ConvertWorker::Loop(Decoder *decoder, Verifier *verifier, FormatCon
 	Format		 format	     = trackToConvert.GetFormat();
 
 	Int64		 trackOffset = encoder->GetEncodedSamples();
-	UnsignedLong	 samplesSize = 512;
+	UnsignedLong	 samplesSize = format.rate / 8;
 
 	trackStartTicks = S::System::System::Clock();
 	trackPosition	= 0;
 
-	waiting		= False;
+	Threads::Access::Set(waiting, False);
 
 	Int			 bytesPerSample = format.bits / 8 * format.channels;
 	Buffer<UnsignedByte>	 buffer(samplesSize * bytesPerSample);
 
-	while (!cancel)
+	while (!Threads::Access::Value(cancel))
 	{
 		Int	 step = samplesSize;
 
@@ -565,8 +574,8 @@ Int64 freac::ConvertWorker::Loop(Decoder *decoder, Verifier *verifier, FormatCon
 		 */
 		Int	 bytes = decoder->Read(buffer);
 
-		if	(bytes == -1) { cancel = True; break; }
-		else if (bytes ==  0)		       break;
+		if	(bytes == -1) { Threads::Access::Set(cancel, True); break; }
+		else if (bytes ==  0)					    break;
 
 		/* Pass samples to verifier.
 		 */
@@ -579,7 +588,7 @@ Int64 freac::ConvertWorker::Loop(Decoder *decoder, Verifier *verifier, FormatCon
 
 		/* Pass samples to encoder.
 		 */
-		if (encoder->Write(buffer) == -1) { cancel = True; break; }
+		if (encoder->Write(buffer) == -1) { Threads::Access::Set(cancel, True); break; }
 
 		/* Update position info.
 		 */
@@ -592,17 +601,17 @@ Int64 freac::ConvertWorker::Loop(Decoder *decoder, Verifier *verifier, FormatCon
 		{
 			BoCA::Utilities::WarningMessage("CD ripping timeout after %1 seconds. Skipping track.", String::FromInt(ripperTimeout));
 
-			cancel = True;
+			Threads::Access::Set(cancel, True);
 		}
 
 		/* Pause if requested.
 		 */
-		while (pause && !cancel) S::System::System::Sleep(50);
+		while (Threads::Access::Value(pause) && !Threads::Access::Value(cancel)) S::System::System::Sleep(50);
 	}
 
 	/* Finish format conversion and processing.
 	 */
-	for (Int i = 0; i < 2 && !cancel; i++)
+	for (Int i = 0; i < 2 && !Threads::Access::Value(cancel); i++)
 	{
 		buffer.Resize(0);
 
@@ -611,13 +620,13 @@ Int64 freac::ConvertWorker::Loop(Decoder *decoder, Verifier *verifier, FormatCon
 
 		if (i == 1 && processor != NIL) processor->Finish(buffer);
 
-		if (encoder->Write(buffer) == -1) cancel = True;
+		if (encoder->Write(buffer) == -1) Threads::Access::Set(cancel, True);
 	}
 
 	/* Inform components about finished/cancelled conversion.
 	 */
-	if (cancel) engine->onCancelTrackConversion.Emit(conversionID, trackToConvert);
-	else	    engine->onFinishTrackConversion.Emit(conversionID, trackToConvert);
+	if (Threads::Access::Value(cancel)) engine->onCancelTrackConversion.Emit(conversionID, trackToConvert);
+	else				    engine->onFinishTrackConversion.Emit(conversionID, trackToConvert);
 
 	/* Compute track length in target format.
 	 */
@@ -645,11 +654,11 @@ Void freac::ConvertWorker::VerifyInput(const String &uri, Verifier *verifier)
 
 	if (verifier->Verify())
 	{
-		log->Write(String("    Successfully verified input ").Append(uriType).Append(": ").Append(uri));
+		log->Write(String("    Successfully verified input ").Append(uriType).Append(": ").Append(Utilities::FormatFileNameForLogging(uri)));
 	}
 	else if (!verifier->GetErrorState())
 	{
-		log->Write(String("    Could not verify input ").Append(uriType).Append(": ").Append(uri), MessageTypeWarning);
+		log->Write(String("    Could not verify input ").Append(uriType).Append(": ").Append(Utilities::FormatFileNameForLogging(uri)), MessageTypeWarning);
 	}
 	else
 	{
@@ -657,7 +666,7 @@ Void freac::ConvertWorker::VerifyInput(const String &uri, Verifier *verifier)
 
 		errorMessage = i18n->TranslateString(String("Failed to verify input ").Append(uriType).Append(": %1"), "Errors").Replace("%1", uri.Contains("://") ? uri : File(uri).GetFileName()).Append("\n\n").Append(verifier->GetErrorString());
 
-		log->Write(String("    Failed to verify input ").Append(uriType).Append(": ").Append(uri), MessageTypeError);
+		log->Write(String("    Failed to verify input ").Append(uriType).Append(": ").Append(Utilities::FormatFileNameForLogging(uri)), MessageTypeError);
 	}
 
 	/* Print messages generated by verifier component.
@@ -671,7 +680,12 @@ Void freac::ConvertWorker::VerifyInput(const String &uri, Verifier *verifier)
 
 	/* Report error message if any.
 	 */
-	if (errorMessage != NIL) onReportError.Emit(errorMessage);
+	if (errorMessage != NIL)
+	{
+		onReportError.Emit(errorMessage);
+
+		Threads::Access::Set(error, True);
+	}
 }
 
 Void freac::ConvertWorker::LogConversionStart(Decoder *decoder, const String &uri, const String &outFile, Bool replace) const
@@ -686,7 +700,7 @@ Void freac::ConvertWorker::LogConversionStart(Decoder *decoder, const String &ur
 	BoCA::Protocol	*log = BoCA::Protocol::Get(logName);
 
 	log->Lock();
-	log->Write(String("    ").Append(verb).Append(": ").Append(uri));
+	log->Write(String("    ").Append(verb).Append(": ").Append(Utilities::FormatFileNameForLogging(uri)));
 
 	/* Log output file.
 	 */
@@ -696,7 +710,9 @@ Void freac::ConvertWorker::LogConversionStart(Decoder *decoder, const String &ur
 
 	if (outFile != NIL && conversionStep != ConversionStepVerify)
 	{
-		log->Write(String("    ").Append(String().FillN(' ', verb.Length() - 2)).Append("to: ").Append(outputFile));
+		String	 basePath = Utilities::GetOutputBasePath(configuration, trackToConvert, outputFile);
+
+		log->Write(String("    ").Append(String().FillN(' ', verb.Length() - 2)).Append("to: ").Append(Utilities::FormatFileNameForLogging(outputFile, basePath)));
 	}
 
 	/* Log overwriting of files.
@@ -718,7 +734,7 @@ Void freac::ConvertWorker::LogConversionStart(Decoder *decoder, const String &ur
 	log->Release();
 }
 
-Void freac::ConvertWorker::LogConversionEnd(const String &uri, Int64 trackLength, UnsignedInt32 rippingCRC, const String &encodeChecksum, const String &verifyChecksum) const
+Void freac::ConvertWorker::LogConversionEnd(const String &uri, Int64 trackLength, UnsignedInt32 rippingCRC, const String &encodeChecksum, const String &verifyChecksum)
 {
 	String	 verb = "converting";
 
@@ -730,19 +746,20 @@ Void freac::ConvertWorker::LogConversionEnd(const String &uri, Int64 trackLength
 
 	log->Lock();
 
+	String	 formattedURI = Utilities::FormatFileNameForLogging(uri);
 	String	 errorMessage;
 
 	if (conversionStep != ConversionStepVerify)
 	{
 		/* Log conversion result.
 		 */
-		if (cancel)
+		if (Threads::Access::Value(cancel))
 		{
-			log->Write(String("    Cancelled ").Append(verb).Append(": ").Append(uri), MessageTypeWarning);
+			log->Write(String("    Cancelled ").Append(verb).Append(": ").Append(formattedURI), MessageTypeWarning);
 		}
 		else
 		{
-			log->Write(String("    Finished ").Append(verb).Append(": ").Append(uri));
+			log->Write(String("    Finished ").Append(verb).Append(": ").Append(formattedURI));
 
 			if (uri.StartsWith("device://") && (conversionStep == ConversionStepOnTheFly ||
 							    conversionStep == ConversionStepDecode)) log->Write(String("        CRC checksum: ").Append(Number(Int64(rippingCRC)).ToHexString(8)));
@@ -752,9 +769,9 @@ Void freac::ConvertWorker::LogConversionEnd(const String &uri, Int64 trackLength
 	{
 		/* Log verification result.
 		 */
-		if (cancel)
+		if (Threads::Access::Value(cancel))
 		{
-			log->Write(String("    Cancelled verifying output file: %1").Replace("%1", uri), MessageTypeWarning);
+			log->Write(String("    Cancelled verifying output file: %1").Replace("%1", formattedURI), MessageTypeWarning);
 		}
 		else if (encodeChecksum != verifyChecksum)
 		{
@@ -762,20 +779,20 @@ Void freac::ConvertWorker::LogConversionEnd(const String &uri, Int64 trackLength
 
 			errorMessage = i18n->TranslateString("Checksum mismatch verifying output file: %1\n\nEncode checksum: %2\nVerify checksum: %3", "Errors").Replace("%1", File(uri).GetFileName()).Replace("%2", encodeChecksum).Replace("%3", verifyChecksum);
 
-			log->Write(String("    Checksum mismatch verifying output file: %1").Replace("%1", uri), MessageTypeError);
+			log->Write(String("    Checksum mismatch verifying output file: %1").Replace("%1", formattedURI), MessageTypeError);
 			log->Write(String("        Encode MD5: %1").Replace("%1", encodeChecksum));
 			log->Write(String("        Verify MD5: %1").Replace("%1", verifyChecksum));
 		}
 		else
 		{
-			log->Write(String("    Successfully verified output file: %1").Replace("%1", uri));
+			log->Write(String("    Successfully verified output file: %1").Replace("%1", formattedURI));
 			log->Write(String("        MD5 hash: %1").Replace("%1", verifyChecksum));
 		}
 	}
 
 	/* Log duration.
 	 */
-	if (!cancel)
+	if (!Threads::Access::Value(cancel))
 	{
 		UnsignedInt64	 ticks	  = S::System::System::Clock() - trackStartTicks;
 		String		 duration = String(ticks / 1000 / 60 % 60 <  10 ?			      "0"  : "").Append(String::FromInt(ticks / 1000 / 60 % 60)).Append(":")
@@ -793,7 +810,12 @@ Void freac::ConvertWorker::LogConversionEnd(const String &uri, Int64 trackLength
 
 	/* Report error message if any.
 	 */
-	if (errorMessage != NIL) onReportError.Emit(errorMessage);
+	if (errorMessage != NIL)
+	{
+		onReportError.Emit(errorMessage);
+
+		Threads::Access::Set(error, True);
+	}
 }
 
 Void freac::ConvertWorker::SetTrackToConvert(const BoCA::Track &nTrack)
@@ -802,29 +824,33 @@ Void freac::ConvertWorker::SetTrackToConvert(const BoCA::Track &nTrack)
 	trackStartTicks	= 0;
 	trackPosition	= 0;
 
-	idle		= False;
-	waiting		= True;
-	error		= False;
+	Threads::Access::Set(idle, False);
+	Threads::Access::Set(waiting, True);
+	Threads::Access::Set(error, False);
+
+	workSignal.Release();
 }
 
 Int freac::ConvertWorker::Pause(Bool value)
 {
-	pause = value;
+	Threads::Access::Set(pause, value);
 
 	return Success();
 }
 
 Int freac::ConvertWorker::Cancel()
 {
-	cancel = True;
+	Threads::Access::Set(cancel, True);
 
 	return Success();
 }
 
 Int freac::ConvertWorker::Quit()
 {
-	cancel = True;
-	quit   = True;
+	Threads::Access::Set(cancel, True);
+	Threads::Access::Set(quit, True);
+
+	workSignal.Release();
 
 	return Success();
 }
