@@ -603,7 +603,7 @@ Error freac::JobConvert::Perform()
 					 */
 					if (deleteFile)
 					{
-						log->Write(String("    Removing source file: ").Append(track.fileName));
+						log->Write(String("    Removing source file: ").Append(Utilities::FormatFileNameForLogging(track.fileName)));
 						log->Write(NIL);
 
 						File(track.fileName).Delete();
@@ -1156,8 +1156,6 @@ Error freac::JobConvert::Perform()
 
 	/* Fill playlist and cuesheet tracks.
 	 */
-	String	 absoluteOutputDir = BoCA::Utilities::GetAbsolutePathName(encoderOutputDirectory);
-
 	foreach (const Track &trackToConvert, tracksToConvert)
 	{
 		Track	 track = convertedTracks.Get(trackToConvert.GetTrackID());
@@ -1174,7 +1172,14 @@ Error freac::JobConvert::Perform()
 			playlistTracks.Add(playlistTrack);
 			cuesheetTracks.Add(playlistTrack);
 
-			track.SaveCoverArtFiles(absoluteOutputDir);
+			/* Save cover art files.
+			 */
+			String	 inputDirectory	 = File(track.fileName).GetFilePath();
+			String	 outputDirectory = encoderOutputDirectory;
+		
+			if (writeToInputDirectory && !track.isCDTrack && BoCA::Utilities::IsFolderWritable(inputDirectory)) outputDirectory = inputDirectory;
+
+			track.SaveCoverArtFiles(BoCA::Utilities::GetAbsolutePathName(outputDirectory));
 		}
 		else
 		{
@@ -1201,6 +1206,7 @@ Error freac::JobConvert::Perform()
 		/* Split playlist tracks to individual playlists.
 		 */
 		Array<String>			 playlistFileNames;
+		Array<String>			 playlistBasePaths;
 
 		Array<Array<Track> *, Void *>	 playlistTrackLists;
 		Array<Array<Track> *, Void *>	 cuesheetTrackLists;
@@ -1209,8 +1215,16 @@ Error freac::JobConvert::Perform()
 		{
 			/* Set playlist filename so it is written to the same place as a single output file.
 			 */
-			if (encodeToSingleFile) playlistFileNames.Add(singleOutFile.Head(singleOutFile.FindLast(".")));
-			else			playlistFileNames.Add(Utilities::GetPlaylistFileName(configuration, playlistTracks.GetFirst(), tracksToConvert));
+			if (encodeToSingleFile)
+			{
+				playlistFileNames.Add(singleOutFile.Head(singleOutFile.FindLast(".")));
+				playlistBasePaths.Add(NIL);
+			}
+			else
+			{
+				playlistFileNames.Add(Utilities::GetPlaylistFileName(configuration, playlistTracks.GetFirst(), tracksToConvert));
+				playlistBasePaths.Add(Utilities::GetPlaylistBasePath(configuration, playlistTracks.GetFirst(), tracksToConvert));
+			}
 
 			playlistTrackLists.Add(new Array<Track>(playlistTracks));
 			cuesheetTrackLists.Add(new Array<Track>(cuesheetTracks));
@@ -1226,6 +1240,8 @@ Error freac::JobConvert::Perform()
 
 				if (playlistFileNames.Add(playlistFileName, playlistFileCRC))
 				{
+					playlistBasePaths.Add(Utilities::GetPlaylistBasePath(configuration, track, tracksToConvert), playlistFileCRC);
+
 					playlistTrackLists.Add(new Array<Track>(), playlistFileCRC);
 					cuesheetTrackLists.Add(new Array<Track>(), playlistFileCRC);
 				}
@@ -1247,7 +1263,7 @@ Error freac::JobConvert::Perform()
 
 			if (playlist != NIL)
 			{
-				log->Write(String("    Writing playlist: ").Append(playlistFileNames.GetNth(i).Append(".").Append(playlistExtension)));
+				log->Write(String("    Writing playlist: ").Append(Utilities::FormatFileNameForLogging(playlistFileNames.GetNth(i).Append(".").Append(playlistExtension), playlistBasePaths.GetNth(i))));
 
 				playlist->SetTrackList(*playlistTrackLists.GetNth(i));
 				playlist->WritePlaylist(String(playlistFileNames.GetNth(i)).Append(".").Append(playlistExtension));
@@ -1261,7 +1277,7 @@ Error freac::JobConvert::Perform()
 
 			if (cuesheet != NIL)
 			{
-				log->Write(String("    Writing cue sheet: ").Append(playlistFileNames.GetNth(i).Append(".cue")));
+				log->Write(String("    Writing cue sheet: ").Append(Utilities::FormatFileNameForLogging(playlistFileNames.GetNth(i).Append(".cue"), playlistBasePaths.GetNth(i))));
 
 				cuesheet->SetTrackList(*cuesheetTrackLists.GetNth(i));
 				cuesheet->WritePlaylist(playlistFileNames.GetNth(i).Append(".cue"));
@@ -1751,6 +1767,8 @@ Void freac::JobConvert::LogSettings(const String &singleOutFile, Int numberOfThr
 	Bool	 enableProcessing	= configuration->GetIntValue(Config::CategoryProcessingID, Config::ProcessingEnableProcessingID, Config::ProcessingEnableProcessingDefault);
 	String	 selectedDSPs		= configuration->GetStringValue(Config::CategoryProcessingID, Config::ProcessingComponentsID, Config::ProcessingComponentsDefault);
 
+	Bool	 logCompletePaths	= configuration->GetIntValue(Config::CategoryLoggingID, Config::LoggingLogCompletePathsID, Config::LoggingLogCompletePathsDefault);
+
 	/* Get encoder properties.
 	 */
 	Registry		&boca	 = Registry::Get();
@@ -1791,14 +1809,17 @@ Void freac::JobConvert::LogSettings(const String &singleOutFile, Int numberOfThr
 
 	if (encodeToSingleFile)
 	{
-		log->Write(String("        Output file:      ").Append(singleOutFile));
+		log->Write(String("        Output file:      ").Append(Utilities::FormatFileNameForLogging(singleOutFile)));
 		log->Write(NIL);
 	}
 	else
 	{
-		log->Write(String("        Output folder:    ").Append(writeToInputDirectory ? String("Using input file folders") : encoderOutputDirectory));
+		if (logCompletePaths)
+		{
+			log->Write(String("        Output folder:    ").Append(writeToInputDirectory ? String("Using input file folders") : encoderOutputDirectory));
 
-		if (writeToInputDirectory) log->Write(String("            Fallback:     ").Append(encoderOutputDirectory));
+			if (writeToInputDirectory) log->Write(String("            Fallback:     ").Append(encoderOutputDirectory));
+		}
 
 		log->Write(String("        Filename pattern: ").Append(filenamePattern));
 		log->Write(NIL);
@@ -1866,7 +1887,7 @@ Void freac::JobConvert::LogCDInfo() const
 		const MCDI	&mcdi	= info->GetNthDeviceMCDI(drive);
 
 		log->Write(String("    Using drive:          ").Append("device://cdda:").Append(String::FromInt(drive)).Append("/"));
-		log->Write(String("        Drive model:      ").Append(device.vendor).Append(" ").Append(device.model).Append(" ").Append(device.revision));
+		log->Write(String("        Drive model:      ").Append(device.GetName()));
 
 #ifdef __WIN32__
 		log->Write(String("        SCSI address:     ").Append(device.path));
@@ -1874,7 +1895,7 @@ Void freac::JobConvert::LogCDInfo() const
 		log->Write(String("        Device path:      ").Append(device.path));
 #endif
 
-		if (configuration->GetIntValue(Config::CategoryRipperID, String("UseOffsetDrive").Append(String::FromInt(drive)), 0)) log->Write(String("        Read offset:      ").Append(String::FromInt(configuration->GetIntValue(Config::CategoryRipperID, String("ReadOffsetDrive").Append(String::FromInt(drive)), 0))).Append(" samples"));
+		if (configuration->GetIntValue(Config::CategoryRipperID, String("UseOffsetDrive-").Append(device.GetID()), 0)) log->Write(String("        Read offset:      ").Append(String::FromInt(configuration->GetIntValue(Config::CategoryRipperID, String("ReadOffsetDrive-").Append(device.GetID()), 0))).Append(" samples"));
 
 		log->Write(NIL);
 
@@ -1918,8 +1939,8 @@ Void freac::JobConvert::LogCDInfo() const
 						 .Append("        ").Append(" | ");
 			}
 
-			entry.Append("      ").Append(String().FillN(' ', 5 - Math::Floor(Math::Log10(trackStartSector)))).Append(String::FromInt(trackStartSector)).Append(" | ")
-			     .Append("    ")  .Append(String().FillN(' ', 5 - Math::Floor(Math::Log10(trackEndSector))))  .Append(String::FromInt(trackEndSector));
+			entry.Append("      ").Append(String().FillN(' ', 5 - Math::Floor(Math::Log10(trackStartSector > 0 ? trackStartSector : 1)))).Append(String::FromInt(trackStartSector)).Append(" | ")
+			     .Append("    ")  .Append(String().FillN(' ', 5 - Math::Floor(Math::Log10(			     trackEndSector	 )))).Append(String::FromInt(trackEndSector));
 
 			log->Write(entry);
 		}
