@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -15,7 +15,6 @@
 #include <player.h>
 #include <config.h>
 #include <utilities.h>
-#include <resources.h>
 
 #include <engine/converter.h>
 
@@ -49,7 +48,7 @@
 #include <support/notification.h>
 
 #ifdef __WIN32__
-#	include <smooth/init.win32.h>
+#	include <windows.h>
 #endif
 
 using namespace smooth::GUI::Dialogs;
@@ -246,10 +245,6 @@ freac::freacGUI::freacGUI()
 
 	mainWnd->SetIcon(ImageLoader::Load(String(currentConfig->resourcesPath).Append("icons/freac.png")));
 
-#ifdef __WIN32__
-	mainWnd->SetIconDirect(LoadImageA(hInstance, MAKEINTRESOURCEA(IDI_ICON), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE | LR_SHARED));
-#endif
-
 	mainWnd->onChangePosition.Connect(&freacGUI::OnChangePosition, this);
 	mainWnd->onChangeSize.Connect(&freacGUI::OnChangeSize, this);
 
@@ -523,7 +518,7 @@ Void freac::freacGUI::ConfigureComponent(const String &id)
 
 		dlg.SetParentWindow(mainWnd);
 
-		if (dlg.ShowDialog() == Error()) BoCA::Utilities::ErrorMessage("No configuration dialog available for:\n\n%1", component->GetName());
+		if (dlg.ShowDialog() == Error()) BoCA::Utilities::InfoMessage("No configuration dialog available for:\n\n%1", component->GetName());
 
 		boca.DeleteComponent(component);
 	}
@@ -570,9 +565,7 @@ Void freac::freacGUI::OnChangeConfiguration()
 	BoCA::Config	*config  = BoCA::Config::Get();
 	BoCA::I18n	*i18n	 = BoCA::I18n::Get();
 
-	Surface		*surface = mainWnd->GetDrawSurface();
-
-	surface->StartPaint(mainWnd->GetVisibleArea());
+	mainWnd->SetVisibleDirect(False);
 
 	/* Update language and menu bars.
 	 */
@@ -616,7 +609,6 @@ Void freac::freacGUI::OnChangeConfiguration()
 				Config::SettingsShowJobsTabDefault)) tabs_main->Add(tab_layer_threads);
 
 	tabs_main->SelectTab(selectedTab);
-	tabs_main->Paint(SP_PAINT);
 
 	/* Do not use tab widget if there would be only one tab.
 	 */
@@ -654,8 +646,6 @@ Void freac::freacGUI::OnChangeConfiguration()
 				tabs_main->Add(tab);
 			}
 		}
-
-		mainWnd->Paint(SP_PAINT);
 	}
 
 	/* Update joblist layer.
@@ -668,9 +658,19 @@ Void freac::freacGUI::OnChangeConfiguration()
 	ToggleSignalProcessing();
 	ToggleUseInputDirectory();
 
-	surface->EndPaint();
-
 	config->SaveSettings();
+
+	mainWnd->SetVisibleDirect(True);
+
+	/* Redraw window contents.
+	 */
+	Surface		*surface = mainWnd->GetDrawSurface();
+
+	surface->StartPaint(mainWnd->GetVisibleArea());
+
+	mainWnd->Paint(SP_PAINT);
+
+	surface->EndPaint();
 }
 
 Void freac::freacGUI::OnDriveChange()
@@ -750,7 +750,7 @@ Void freac::freacGUI::QueryCDDB()
 		const Track	&track = joblist->GetNthTrack(i);
 		const Info	&info  = track.GetInfo();
 
-		if (info.mcdi.GetData().Size() > 0)
+		if (info.mcdi.IsValid())
 		{
 			String	 queryString = CDDB::QueryStringFromMCDI(info.mcdi);
 
@@ -826,7 +826,7 @@ Void freac::freacGUI::QueryCDDB()
 			Track	 track = joblist->GetNthTrack(j);
 			Info	 info  = track.GetInfo();
 
-			if ((info.mcdi.GetData().Size() > 0 && queryString == CDDB::QueryStringFromMCDI(info.mcdi)) ||
+			if ((info.mcdi.IsValid() && queryString == CDDB::QueryStringFromMCDI(info.mcdi)) ||
 			    (info.offsets != NIL && queryString == CDDB::QueryStringFromOffsets(info.offsets)))
 			{
 				Int	 trackNumber = -1;
@@ -1058,7 +1058,7 @@ Void freac::freacGUI::FillMenus()
 		for (Int i = 0; i < info->GetNumberOfDevices(); i++)
 		{
 			const Device	&device = info->GetNthDeviceInfo(i);
-			String		 name	= String(device.vendor).Append(" ").Append(device.model).Append(" ").Append(device.revision).Trim();
+			String		 name	= device.GetName();
 
 			menu_drives->AddEntry(name, &clicked_drive, i)->onAction.Connect(&freacGUI::ReadCD, this);
 			menu_seldrive->AddEntry(name, &config->GetPersistentIntValue(Config::CategoryRipperID, Config::RipperActiveDriveID, Config::RipperActiveDriveDefault), i);
