@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -272,7 +272,7 @@ Void freac::Utilities::UpdateGenreList(List *list, const String &genre)
 	FillGenreList(list);
 }
 
-String freac::Utilities::GetOutputFileName(BoCA::Config *config, const Track &track)
+String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Track &track)
 {
 	if (track.outputFile != NIL) return track.outputFile;
 
@@ -282,26 +282,11 @@ String freac::Utilities::GetOutputFileName(BoCA::Config *config, const Track &tr
 
 	/* Check if input file folder is writable.
 	 */
-	Int	 lastBs	 = Math::Max(track.fileName.FindLast("\\"), track.fileName.FindLast("/"));
-	Int	 lastDot = track.fileName.FindLast(".");
+	String	 shortInFileName = File(track.fileName).GetFileName().Head(File(track.fileName).GetFileName().FindLast("."));
+	String	 inFileDirectory = File(track.fileName).GetFilePath();
+	Bool	 writeToInputDir = config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault);
 
-	if (lastDot < lastBs) lastDot = track.fileName.Length();
-
-	String	 shortInFileName = track.fileName.SubString(lastBs + 1, lastDot - lastBs - 1);
-	String	 inFileDirectory = track.fileName.Head(lastBs);
-	Bool	 writeToInputDir = False;
-
-	if (config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault) && !track.isCDTrack)
-	{
-		String		 file = String(inFileDirectory).Append(Directory::GetDirectoryDelimiter()).Append(String::FromInt(S::System::System::Clock())).Append(".temp");
-		OutStream	 temp(STREAM_FILE, file, OS_REPLACE);
-
-		if (temp.GetLastError() == IO_ERROR_OK) writeToInputDir = True;
-
-		temp.Close();
-
-		File(file).Delete();
-	}
+	if (writeToInputDir && (track.isCDTrack || !BoCA::Utilities::IsFolderWritable(inFileDirectory))) writeToInputDir = False;
 
 	/* Construct output filename.
 	 */
@@ -341,11 +326,11 @@ String freac::Utilities::GetOutputFileName(BoCA::Config *config, const Track &tr
 		String		 shortOutFileName = filePattern;
 
 		DateTime	 currentDateTime  = DateTime::Current();
-		String		 currentDate	  = String().FillN('0', 3 - Math::Floor(Math::Log10(currentDateTime.GetYear()))).Append(String::FromInt(currentDateTime.GetYear()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMonth())))).Append(String::FromInt(currentDateTime.GetMonth()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetDay())))).Append(String::FromInt(currentDateTime.GetDay()));
-		String		 currentTime	  = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()))).Append(String::FromInt(currentDateTime.GetHour()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute())))).Append(String::FromInt(currentDateTime.GetMinute()));
+		String		 currentDate	  = String().FillN('0', 3 - Math::Floor(Math::Log10(				      currentDateTime.GetYear()	     ))) .Append(String::FromInt(currentDateTime.GetYear()))
+					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetMonth()     )))).Append(String::FromInt(currentDateTime.GetMonth()))
+					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetDay()	     )))).Append(String::FromInt(currentDateTime.GetDay()));
+		String		 currentTime	  = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()	> 0 ? currentDateTime.GetHour()	  : 1))) .Append(String::FromInt(currentDateTime.GetHour()))
+					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute() > 0 ? currentDateTime.GetMinute() : 1)))).Append(String::FromInt(currentDateTime.GetMinute()));
 
 		shortOutFileName.Replace("<artist>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")));
 		shortOutFileName.Replace("<title>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.title.Length() > 0 ? info.title : i18n->TranslateString("unknown title")));
@@ -476,7 +461,7 @@ String freac::Utilities::GetOutputFileName(BoCA::Config *config, const Track &tr
 	return outputFileName;
 }
 
-String freac::Utilities::GetSingleOutputFileName(BoCA::Config *config, const Array<Track> &tracks)
+String freac::Utilities::GetSingleOutputFileName(const BoCA::Config *config, const Array<Track> &tracks)
 {
 	/* Check if an output filename has already been set.
 	 */
@@ -540,13 +525,13 @@ String freac::Utilities::GetSingleOutputFileName(BoCA::Config *config, const Arr
 
 		/* Save selected path.
 		 */
-		config->SetStringValue(Config::CategorySettingsID, Config::SettingsLastSelectedSaveDirID, File(singleOutputFileName).GetFilePath());
+		BoCA::Config::Get()->SetStringValue(Config::CategorySettingsID, Config::SettingsLastSelectedSaveDirID, File(singleOutputFileName).GetFilePath());
 	}
 
 	return singleOutputFileName;
 }
 
-String freac::Utilities::GetSingleOutputFileNameDefault(BoCA::Config *config, const Array<Track> &tracks)
+String freac::Utilities::GetSingleOutputFileNameDefault(const BoCA::Config *config, const Array<Track> &tracks)
 {
 	/* Get configuration.
 	 */
@@ -585,7 +570,7 @@ String freac::Utilities::GetSingleOutputFileNameDefault(BoCA::Config *config, co
 					  .Append(BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length()  > 0 ? info.album  : i18n->TranslateString("unknown album")))).Append(fileExtension != NIL ? "." : NIL).Append(fileExtension);
 }
 
-String freac::Utilities::GetPlaylistFileName(BoCA::Config *config, const Track &track, const Array<Track> &originalTracks)
+String freac::Utilities::GetPlaylistFileName(const BoCA::Config *config, const Track &track, const Array<Track> &originalTracks)
 {
 	BoCA::I18n	*i18n = BoCA::I18n::Get();
 
@@ -597,22 +582,13 @@ String freac::Utilities::GetPlaylistFileName(BoCA::Config *config, const Track &
 
 	/* Find playlist output folder.
 	 */
-	const Track	&originalTrack = originalTracks.Get(track.GetTrackID());
+	const Track	&originalTrack	 = originalTracks.Get(track.GetTrackID());
 
-	String		 outputDir     = config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault);
-	String		 inputDir      = originalTrack.fileName.Head(Math::Max(originalTrack.fileName.FindLast("\\"), originalTrack.fileName.FindLast("/")) + 1);
+	String		 inputDir	 = File(originalTrack.fileName).GetFilePath().Append(Directory::GetDirectoryDelimiter());
+	String		 outputDir	 = config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault);
+	Bool		 writeToInputDir = config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault);
 
-	if (config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault) && !originalTrack.isCDTrack)
-	{
-		String		 file = String(inputDir).Append(String::FromInt(S::System::System::Clock())).Append(".temp");
-		OutStream	 temp(STREAM_FILE, file, OS_REPLACE);
-
-		if (temp.GetLastError() == IO_ERROR_OK) outputDir = inputDir;
-
-		temp.Close();
-
-		File(file).Delete();
-	}
+	if (writeToInputDir && !originalTrack.isCDTrack && BoCA::Utilities::IsFolderWritable(inputDir)) outputDir = inputDir;
 
 	/* Generate playlist file name.
 	 */
@@ -629,11 +605,11 @@ String freac::Utilities::GetPlaylistFileName(BoCA::Config *config, const Track &
 		if (shortOutFileName.Trim() == NIL) shortOutFileName = Config::PlaylistFilenamePatternDefault;
 
 		DateTime	 currentDateTime  = DateTime::Current();
-		String		 currentDate	  = String().FillN('0', 3 - Math::Floor(Math::Log10(currentDateTime.GetYear()))).Append(String::FromInt(currentDateTime.GetYear()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMonth())))).Append(String::FromInt(currentDateTime.GetMonth()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetDay())))).Append(String::FromInt(currentDateTime.GetDay()));
-		String		 currentTime	  = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()))).Append(String::FromInt(currentDateTime.GetHour()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute())))).Append(String::FromInt(currentDateTime.GetMinute()));
+		String		 currentDate	  = String().FillN('0', 3 - Math::Floor(Math::Log10(				      currentDateTime.GetYear()      ))) .Append(String::FromInt(currentDateTime.GetYear()))
+					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetMonth()     )))).Append(String::FromInt(currentDateTime.GetMonth()))
+					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetDay()	     )))).Append(String::FromInt(currentDateTime.GetDay()));
+		String		 currentTime	  = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()	> 0 ? currentDateTime.GetHour()	  : 1))) .Append(String::FromInt(currentDateTime.GetHour()))
+					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute()	> 0 ? currentDateTime.GetMinute() : 1)))).Append(String::FromInt(currentDateTime.GetMinute()));
 
 		shortOutFileName.Replace("<artist>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")));
 		shortOutFileName.Replace("<album>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length() > 0 ? info.album : i18n->TranslateString("unknown album")));
@@ -688,6 +664,62 @@ String freac::Utilities::GetPlaylistFileName(BoCA::Config *config, const Track &
 	return BoCA::Utilities::NormalizeFileName(playlistFileName);
 }
 
+String freac::Utilities::FormatFileNameForLogging(const String &fileName, const String &basePath)
+{
+	BoCA::Config	*config	= BoCA::Config::Get();
+
+	Bool	 logCompletePaths = config->GetIntValue(Config::CategoryLoggingID, Config::LoggingLogCompletePathsID, Config::LoggingLogCompletePathsDefault);
+
+	if (logCompletePaths || fileName.Contains("://")) return fileName;
+
+	String	 fixedBasePath	   = basePath;
+	String	 shortenedFileName = fileName;
+
+	if (fixedBasePath != NIL && !fixedBasePath.EndsWith(Directory::GetDirectoryDelimiter())) fixedBasePath.Append(Directory::GetDirectoryDelimiter());
+
+	if (fixedBasePath != NIL && fileName.StartsWith(fixedBasePath)) shortenedFileName = fileName.Tail(fileName.Length() - fixedBasePath.Length());
+	else								shortenedFileName = File(fileName).GetFileName();
+
+	return shortenedFileName;
+}
+
+String freac::Utilities::GetOutputBasePath(const BoCA::Config *config, const Track &track, const String &outputFile)
+{
+	Bool	 encodeToSingleFile	= config->GetIntValue(Config::CategorySettingsID, Config::SettingsEncodeToSingleFileID, Config::SettingsEncodeToSingleFileDefault);
+
+	if (encodeToSingleFile) return NIL;
+
+	String	 encoderOutputDirectory	= config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault);
+	Bool	 writeToInputDirectory	= config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault);
+
+	String	 basePath  = encoderOutputDirectory;
+	String	 inputPath = track.fileName.StartsWith("device://") ? String() : File(track.fileName).GetFilePath();
+
+	if (writeToInputDirectory && inputPath != NIL && outputFile.StartsWith(inputPath)) basePath = inputPath;
+
+	return basePath;
+}
+
+String freac::Utilities::GetPlaylistBasePath(const BoCA::Config *config, const Track &track, const Array<Track> &originalTracks)
+{
+	Bool	 encodeToSingleFile	 = config->GetIntValue(Config::CategorySettingsID, Config::SettingsEncodeToSingleFileID, Config::SettingsEncodeToSingleFileDefault);
+
+	if (encodeToSingleFile) return NIL;
+
+	String	 encoderOutputDirectory	 = config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault);
+	Bool	 writeToInputDirectory	 = config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault);
+
+	const Track	&originalTrack	 = originalTracks.Get(track.GetTrackID());
+	String		 inputDirectory	 = File(originalTrack.fileName).GetFilePath().Append(Directory::GetDirectoryDelimiter());
+
+	if (writeToInputDirectory && !originalTrack.isCDTrack && BoCA::Utilities::IsFolderWritable(inputDirectory)) encoderOutputDirectory = inputDirectory;
+
+	String	 playlistOutputDirectory = config->GetStringValue(Config::CategoryPlaylistID, Config::PlaylistOutputDirID, encoderOutputDirectory);
+	Bool	 useEncoderOutputDir	 = config->GetIntValue(Config::CategoryPlaylistID, Config::PlaylistUseEncoderOutputDirID, Config::PlaylistUseEncoderOutputDirDefault);
+
+	return BoCA::Utilities::GetAbsolutePathName(useEncoderOutputDir ? encoderOutputDirectory : playlistOutputDirectory);
+}
+
 Bool freac::Utilities::SetProcessPriority()
 {
 #ifdef __WIN32__
